package io.humanteq.hqm_flutter_plugin

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.humanteq.hqsdkapps.InstalledApplicationsCollector
import io.humanteq.hqsdkcore.HQSdk
import android.content.Context

class HqmFlutterPlugin(val context: Context) : MethodCallHandler {
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "hqm_flutter_plugin")
            channel.setMethodCallHandler(HqmFlutterPlugin(registrar.context()))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "enableDebug" -> {
                val isEnable: Boolean = call.argument("enableDebug")!!

                android.util.Log.d("HQM_Flutter", "Debug mode ${if (isEnable) "enabled" else "disabled"}")
                HQSdk.enableDebug(isEnable)
                result.success(true)
            }
            "isInitialized" -> {
                val isInitialized = HQSdk.isInitialized
                android.util.Log.d("HQM_Flutter", "HQM SDK is ${if (!isInitialized) "not " else ""}initialized.")

                if (isInitialized) result.success(true)
                else result.success(false)
            }
            "getTestGroup" -> {
                val groupName: String = call.argument("groupName")!!
                val limit: Long = call.argument("limit")!!

                android.util.Log.d("HQM_Flutter", "Generating group for '$groupName'...")
                val groupId = HQSdk.getTestGroup(groupName, limit)
                android.util.Log.d("HQM_Flutter", "Group generated for '$groupName': $groupId")
                result.success(groupId)
            }
            "collectApps" -> {
                android.util.Log.d("HQM_Flutter", "Collecting apps...")
                HQSdk.start(InstalledApplicationsCollector())
                result.success(true)
            }
            "init" -> {
                val apiKey: String = call.argument("apiKey")!!
                val runOnlyInForeground: Boolean = call.argument("runOnlyInForeground")!!
                val allowSlackDebug: Boolean = call.argument("allowSlackDebug")!!

                android.util.Log.d("HQM_Flutter", "Initializing HQM... " +
                        "runOnlyInForeground: $runOnlyInForeground " +
                        "allowSlackDebug: $allowSlackDebug")

                HQSdk.init(
                        context.applicationContext,
                        apiKey,
                        runOnlyInForeground,
                        allowSlackDebug
                )?.invokeOnCompletion {
                    if (it == null) {
                        android.util.Log.d("HQM_Flutter", "HQM initialized successfully.")
                        result.success(true)
                    } else {
                        it.printStackTrace()
                        android.util.Log.d("HQM_Flutter", "HQM initialization failed.")
                        result.error("Fail", "${it.message}", false);
                    }
                }
            }
            else -> result.notImplemented()
        }
    }
}
